<?php

abstract class Byte_Mavo_Plugin_Abstract
{
    protected $_results = array();
    protected $_magentoPath = '';
    protected $_argv = array();
    protected $_longOpts = array();
    protected $_shortOpts = array();

    // Need to be overridden in child classes

    /**
     * Stub output
     *
     * @var string
     */
    protected $_stub = '';

    /**
     * Stub output. The array may be used to put multiple stubs in, and a random one will
     * be returned. If the array is empty, $_stub will be returned.
     *
     * @var array
     */
    protected $_stubs = array();

    /**
     * Name of the plugin; defaults to the filename without '.php'
     *
     * @var string
     */
    protected $_name;


    public function __construct()
    {
        set_error_handler(array('Byte_Mavo_Plugin_Abstract', 'errorHandler'));

        if (!empty($argv)) {
            $this->_argv = $argv;
        }
        if (isset($_SERVER['argv'])) {
            $this->_argv = $_SERVER['argv'];
        }
        if (empty($this->_name)) {
            if (!empty($this->_argv[0])) {
                $this->_name = basename($this->_argv[0], '.php');
            } else {
                $this->_name = 'default';
            }
        }

        $this->_addLongOpt('magentopath:');
        $this->_addLongOpt('stub');
        $this->_addLongOpt('debug');

        $opt = $this->getOpt();
        if (isset($opt['debug'])) {
            error_reporting(E_ALL);
        } else {
            ob_start();
            ini_set('display_errors', 0);
        }

        // Disable output compression
        if(ini_get('zlib.output_compression')){
            ini_set('zlib.output_compression', 'Off');
        }

        $this->_construct();
    }

    protected function _construct()
    {
        // empty; can be overridden in child classes
    }

    /**
     * Get the result of getopt() on the possible options
     *
     * Just like, getopt(), returns FALSE if an option without value is given (e.g. --skip_check)
     *
     * @see _addLongOpt()
     * @see _addShortOpt()
     * @see getopt()
     * @return array
     */
    public function getOpt($key = null, $default = null)
    {
        $options = getopt(implode('', $this->_shortOpts), $this->_longOpts);
        if ($key) {
            if (isset($options[$key])) {
                return $options[$key];
            }
            return $default;
        }
        return $options;
    }


    /**
     * Add a long command line option possibility
     *
     * @see getopt()
     * @param string $name
     * @return Byte_Mavo_Plugin_Abstract
     */
    protected function _addLongOpt($name)
    {
        $this->_longOpts[] = $name;
        $this->_longOpts = array_unique($this->_longOpts);
        return $this;
    }

    /**
     * Add a short command line option possibility
     *
     * @see getopt()
     * @param string $name
     * @return Byte_Mavo_Plugin_Abstract
     */
    protected function _addShortOpt($name)
    {
        $this->_shortOpts[] = $name;
        $this->_shortOpts = array_unique($this->_shortOpts);
        return $this;
    }

    /**
     * Check for the existence of the first argument (being the path to the Magento installation) and set self::_magentoPath
     *
     * @return string  Magento path, stripped from trailing slash
     */
    public function checkMagentoPathArgument()
    {
        $opts = $this->getOpt();

        if (empty($opts['magentopath'])) {
            $this->dieError('No Magento path given. Use --magentopath=xx');
        }

        $magentoPath = rtrim($opts['magentopath'], '/');

        if (!is_dir($magentoPath)) {
            $this->dieError('Magento path does not exist');
        }
        if (!file_exists($magentoPath . '/app/etc/local.xml') || !file_exists($magentoPath . '/app/Mage.php')) {
            $this->dieError('Magento path does not seem to point to a valid Magento installation');
        }

        $this->_magentoPath = realpath($magentoPath);
        return $this->_magentoPath;
    }

    /**
     * Die with an error message
     *
     * @param string $message
     * @return never :)
     */
    public static function dieError($message)
    {
        self::_output(array('error' => $message));
        exit(0);
    }

    /**
     * Add a result to the output
     *
     * @param string $name
     * @param mixed  $result
     */
    public function addResult($name, $result)
    {
        if (empty($name)) {
            $name = $this->_name;
        }
        if (is_scalar($result) || is_null($result)) {
            $result = array('result' => $result);
        }
        $this->_results[$name] = $result;
    }

    /**
     * Set up a Magento environment, defaults to admin
     *
     * @return void
     */
    public function createMagentoEnvironment($runCode = 'admin')
    {
        $this->checkMagentoPathArgument();

        try {
            require_once $this->_magentoPath . '/app/Mage.php';

            if (!Mage::isInstalled()) {
                $this->dieError('Magento is not installed yet, please complete install wizard first.');
            }

            Mage::app($runCode);
        } catch (Exception $e) {
            $this->dieError('Cannot create Magento environment: ' . $e->getMessage());
        }
    }


    public function ensureMagentoCommunityVersion($version)
    {
        // this must be called *after* running app(), because otherwise the config is not initialized
        if (!class_exists('Mage', false)) {
            $this->createMagentoEnvironment();
        }
        $runningVersion = $this->getMagentoCommunityVersion();
        if (version_compare($runningVersion, $version, '<')) {
            $this->dieError('Unsupported Magento version: ' . $this->getMagentoVersionString());
        }
    }


    /**
     * Checks whether the current Magento Edition is Enterprise
     *
     * @return bool
     */
    public function isMagentoEnterprise()
    {
        if (method_exists('Mage', 'getEdition')) {
            if (Mage::getEdition() == Mage::EDITION_ENTERPRISE) {
                return true;
            } else {
                return false;
            }
        }
        return Mage::getConfig()->getModuleConfig('Enterprise_Enterprise')
            && Mage::getConfig()->getModuleConfig('Enterprise_AdminGws')
            && Mage::getConfig()->getModuleConfig('Enterprise_Checkout')
            && Mage::getConfig()->getModuleConfig ('Enterprise_Customer');
    }

    /**
     * Checks whether the current Magento Edition is Professional
     *
     * @return bool
     */
    public function isMagentoProfessional()
    {
        if (method_exists('Mage', 'getEdition')) {
            if (Mage::getEdition() == Mage::EDITION_PROFESSIONAL) {
                return true;
            } else {
                return false;
            }
        }
        return Mage::getConfig()->getModuleConfig('Enterprise_Enterprise')
            && !Mage::getConfig()->getModuleConfig('Enterprise_AdminGws')
            && !Mage::getConfig()->getModuleConfig('Enterprise_Checkout')
            && !Mage::getConfig ()->getModuleConfig('Enterprise_Customer');
    }

    /**
     * Checks whether the current Magento Edition is Community
     *
     * @return bool
     */
    public function isMagentoCommunity()
    {
        if (method_exists('Mage', 'getEdition')) {
            if (Mage::getEdition() == Mage::EDITION_COMMUNITY) {
                return true;
            } else {
                return false;
            }
        }
        return !$this->isMagentoEnterprise() && !$this->isMagentoProfessional();
    }

    /**
     * Get the Magento version string, including edition
     *
     * @return string i.e. 'Community 1.6.2.0' or 'Professional 1.11.0.0'
     */
    public function getMagentoVersionString()
    {
        $edition = null;
        if (method_exists('Mage', 'getEdition')) {
            $edition = Mage::getEdition();
        }
        if (!$edition && $this->isMagentoEnterprise()) {
            $edition = 'Enterprise';
        }
        if (!$edition && $this->isMagentoProfessional()) {
            $edition = 'Professional';
        }
        if (!$edition) {
            $edition = 'Community';
        }
        return $edition . ' ' . Mage::getVersion();
    }

    /**
     * Attempt to convert an Enterprise, Professional, Community Magento version number to its compatible Community version
     *
     * @return string
     */
    public function getMagentoCommunityVersion()
    {
        $version = Mage::getVersion();

        $edition = null;
        if (method_exists('Mage', 'getEdition')) {
            $edition = Mage::getEdition();
        }

        if ($edition && $edition == Mage::EDITION_COMMUNITY) {
            return $version;
        }

        if ($this->isMagentoEnterprise()) {
            if (version_compare($version, '1.13.0', '>=' )) {
                return '1.8.0';
            } elseif (version_compare($version, '1.12.0', '>=' )) {
                return '1.7.0';
            } elseif (version_compare($version, '1.11.0', '>=' )) {
                return '1.6.0';
            } elseif (version_compare($version, '1.9.1', '>=' )) {
                return '1.5.0';
            } elseif (version_compare($version, '1.9.0', '>=' )) {
                return '1.4.2';
            } elseif (version_compare($version, '1.8.0', '>=' )) {
                return '1.3.1';
            } else {
                return '1.3.1';
            }
        }

        if ($this->isMagentoProfessional()) {
            // Magento Professional is discontinued after 1.11
            if (version_compare($version, '1.11.0', '>=')) {
                return '1.6.0';
            } elseif (version_compare($version, '1.9.0', '>=')) {
                return '1.5.0';
            } elseif (version_compare($version, '1.8.0', '>=')) {
                return '1.4.1';
            } elseif (version_compare($version, '1.7.0', '>=')) {
                return '1.3.1';
            } else {
                return '1.3.1';
            }
        }

        return $version;
    }

    /**
     * Output all results to stdout
     *
     * @return void
     */
    public function outputResult()
    {
        $this->_output($this->_results);
    }

    /**
     * Output a string to stdout
     *
     * If the string is not a string, it will first be json_encode'd
     * The command line argument 'pretty' causes a (more) readable output
     *
     * @param mixed  $output  The output, a string is assumed to be json
     * @return void
     */
    protected static function _output($output)
    {
        if (is_string($output)) {
            $json = $output;
        } else {
            $json = json_encode($output);
        }
        ob_end_clean();
        echo self::_jsonPrettify($json) . "\n";
    }


    /**
     * Return a readable json string
     *
     * @param string $json
     * @return string
     */
    protected static function _jsonPrettify($json)
    {
        $result      = '';
        $pos         = 0;
        $strLen      = strlen($json);
        $indentStr   = '  ';
        $newLine     = "\n";
        $prevChar    = '';
        $outOfQuotes = true;

        for ($i=0; $i<=$strLen; $i++) {
            // Grab the next character in the string.
            $char = substr($json, $i, 1);

            // Are we inside a quoted string?
            if ($char == '"' && $prevChar != '\\') {
                $outOfQuotes = !$outOfQuotes;

                // If this character is the end of an element,
                // output a new line and indent the next line.
            } else if(($char == '}' || $char == ']') && $outOfQuotes) {
                $result .= $newLine;
                $pos --;
                for ($j=0; $j<$pos; $j++) {
                    $result .= $indentStr;
                }
            }

            // Add the character to the result string.
            $result .= $char;

            // If the last character was the beginning of an element,
            // output a new line and indent the next line.
            if (($char == ',' || $char == '{' || $char == '[') && $outOfQuotes) {
                $result .= $newLine;
                if ($char == '{' || $char == '[') {
                    $pos ++;
                }

                for ($j = 0; $j < $pos; $j++) {
                    $result .= $indentStr;
                }
            }

            $prevChar = $char;
        }
        return $result;
    }


    protected function _setMagentoIncludePath()
    {
        $this->checkMagentoPathArgument();
        if (!class_exists('Mage')) {
            define('DS', DIRECTORY_SEPARATOR);
            define('PS', PATH_SEPARATOR);
            define('BP', $this->_magentoPath);

            $paths = array(
                BP . DS . 'app' . DS . 'code' . DS . 'local',
                BP . DS . 'app' . DS . 'code' . DS . 'community',
                BP . DS . 'app' . DS . 'code' . DS . 'core',
                BP . DS . 'lib',
            );

            $originalIncludePath = get_include_path();
            $appPath = implode(PS, $paths);
            set_include_path($appPath . PS . $originalIncludePath);
        }
    }


    /**
     * Runs the plugin, does stub handling
     *
     * @return void
     */
    public function run()
    {
        if (extension_loaded('newrelic')) {
            newrelic_ignore_transaction();
        }

        $options = $this->getOpt();
        if (isset($options['stub'])) {
            // use encode & decode to make sure it's correct json
            ob_end_clean();
            echo $this->_jsonPrettify(json_encode(json_decode($this->_getStub()))) . "\n";
            exit;
        }
    }


    /**
     * Returns the stub output for this plugin
     *
     * @return string
     */
    protected function _getStub()
    {
        if (count($this->_stubs) > 0) {
            return $this->_stubs[array_rand($this->_stubs, 1)];
        }
        else {
            return $this->_stub;
        }
    }


    /**
     * Handles error for 'regular' Magento errors
     *
     * @see set_error_handler()
     * @param int    $errno
     * @param string $errstr
     * @param string $errfile
     * @param int    $errline
     * @param array  $errcontext
     */
    public static function errorHandler($errno, $errstr, $errfile = '', $errline = 0, $errcontext = array())
    {
        if (!(error_reporting() & $errno)) {
            // This error code is not included in error_reporting
            return;
        }

        $message = $errstr;

        if ($errfile) {
            $message .= ' in ' . $errfile;
        }
        if ($errline) {
            $message .= ' on line ' . $errline;
        }
        self::_output(array('error' => 'PHP Error: ' . $message));
        exit(1);
    }

}
