#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Cache plugin (performance)
 * ==========================
 *
 * Returns all caches known to Magento and their status.
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s):
 * - caches
 *
 * For every known cache in Magento, it outputs the following properties:
 * - string  code         To refer to this cache
 * - bool    enabled
 * - string  name
 * - string  description  Typically, but necessarily, longer than name
 *
 * Interpreting
 * ------------
 * There is no 'good' or 'bad' here. Of course, for performance it's best
 * to have all caches enabled so it would be helpful to list the disabled
 * caches.
 */
class Byte_Mavo_Plugin_Cache extends Byte_Mavo_Plugin_Abstract
{
    // in a typical magento install, there are more caches than in the stub!
    protected $_stub = '{
  "caches":[
    {
      "code":"config",
      "enabled":false,
      "name":"Configuration",
      "description":"System(config.xml, local.xml) and modules configuration files(config.xml)."
    },
    {
      "code":"layout",
      "enabled":false,
      "name":"Layouts",
      "description":"Layout building instructions."
    },
    {
      "code":"block_html",
      "enabled":true,
      "name":"Blocks HTML output",
      "description":"Page blocks HTML."
    },
    {
      "code":"translate",
      "enabled":true,
      "name":"Translations",
      "description":"Translation files."
    },
    {
      "code":"collections",
      "enabled":true,
      "name":"Collections Data",
      "description":"Collection data files."
    },
    {
      "code":"eav",
      "enabled":true,
      "name":"EAV types and attributes",
      "description":"Entity types declaration cache."
    },
    {
      "code":"config_api",
      "enabled":false,
      "name":"Web Services Configuration",
      "description":"Web Services definition files (api.xml)."
    },
    {
      "code":"config_api2",
      "enabled":false,
      "name":"Web Services Configuration",
      "description":"Web Services definition files (api2.xml)."
    }
  ]
}
';

    /** @var Cache types cache */
    protected $_cacheTypes;

    public function run()
    {
        parent::run();

        $this->addResult('caches', $this->getCaches());
        $this->outputResult();
    }


    public function getCaches()
    {
        $this->createMagentoEnvironment();
        $this->ensureMagentoCommunityVersion('1.4.0.0');

        $cacheIds = array_keys($this->_getCacheTypes());
        $allTypes = Mage::app()->useCache();
        foreach ($cacheIds as $code) {
            $cacheType = $this->_getCacheTypeByCode($code);
            $enabled = (isset($allTypes[$code]) && $allTypes[$code]) ? true : false;
            if ($cacheType) {
                $result[] = array(
                    'code'        => $code,
                    'enabled'     => (bool)$enabled,
                    'name'        => $cacheType->getCacheType(),
                    'description' => $cacheType->getDescription()
                );
            }
        }

        return $result;
    }


    protected function _getCacheTypes()
    {
        if (is_null($this->_cacheTypes)) {
            $this->_cacheTypes = Mage::app()->getCacheInstance()->getTypes();
        }
        return $this->_cacheTypes;
    }


    protected function _getCacheTypeByCode($identifier)
    {
        $types = $this->_getCacheTypes();
        if (isset($types[$identifier])) {
            return $types[$identifier];
        }
        return null;
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_Cache();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Cache::dieError('Error running plugin: ' . $e->getMessage());
}
