#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Catalog plugin (informational)
 * =================================
 *
 * Checks Magento catalog metrics
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s)
 * - catalog_metrics  : Metrics about the catalog
 * - store_metrics    : Metrics about the configured stores
 *
 * 'catalog_metrics' is a hash with keys:
 * - product_count               : The total number of products in the database
 * - product_count_non_composite : The number of non composite (individual sellable) products
 * - category_count              : The number of categories in all stores
 * - product_types               : An array with data about the configured product types: type_id, is_composite, label, product_count
 *
 * 'store_metrics' is a hash with keys:
 * - store_count                 : The number of configured frontend stores - includes disabled stores, excludes the admin store
 * - website_count               : The number of configured frontend websites - excludes the admin website
 *
 * Interpreting
 * ------------
 * -
 */
class Byte_Mavo_Plugin_Catalog extends Byte_Mavo_Plugin_Abstract
{
    protected $_stub =
        '{
  "catalog_metrics":{
    "product_types":{
      "simple":{
        "type_id":"simple",
        "is_composite":false,
        "label":"Simple Product",
        "product_count":101
      },
      "grouped":{
        "type_id":"grouped",
        "is_composite":true,
        "label":"Grouped Product",
        "product_count":1
      },
      "configurable":{
        "type_id":"configurable",
        "is_composite":true,
        "label":"Configurable Product",
        "product_count":11
      },
      "virtual":{
        "type_id":"virtual",
        "is_composite":false,
        "label":"Virtual Product",
        "product_count":3
      },
      "bundle":{
        "type_id":"bundle",
        "is_composite":true,
        "label":"Bundle Product",
        "product_count":4
      },
      "downloadable":{
        "type_id":"downloadable",
        "is_composite":false,
        "label":"Downloadable Product",
        "product_count":0
      }
    },
    "product_count":120,
    "product_count_non_composite":104,
    "category_count":27
  },
  "store_metrics":{
    "store_count":6,
    "website_count":2
  }
}';


    public function run()
    {
        parent::run();

        $this->addResult('catalog_metrics', $this->getCatalogMetrics());
        $this->addResult('store_metrics', $this->getStoreMetrics());
        $this->outputResult();
    }


    public function getCatalogMetrics()
    {
        $this->createMagentoEnvironment();
        $result = array();

        $result['product_types'] = array();
        $nonCompositeCount = 0;
        foreach (Mage_Catalog_Model_Product_Type::getTypes() as $typeId => $type) {
            $productCount = Mage::getResourceModel('catalog/product_collection')
                ->addAttributeToFilter('type_id', $typeId)
                ->getSize();

            $result['product_types'][$typeId] = array(
                'type_id'       => $typeId,
                'is_composite'  => !empty($type['composite']) ? (bool)$type['composite'] : false,
                'label'         => !empty($type['label']) ? $type['label'] : $typeId,
                'product_count' => $productCount,
            );

            if (empty($type['composite'])) {
                $nonCompositeCount += $productCount;
            }
        }

        $result['product_count'] = Mage::getResourceModel('catalog/product_collection')
            ->getSize();

        $result['product_count_non_composite'] = $nonCompositeCount;

        $result['category_count'] = Mage::getResourceModel('catalog/category_collection')
            ->getSize();

        return $result;
    }


    public function getStoreMetrics()
    {
        $result = array(
            'store_count'   => count(Mage::app()->getStores(false)),
            'website_count' => count(Mage::app()->getWebsites(false)),
        );

        return $result;
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_Catalog();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Catalog::dieError('Error running plugin: ' . $e->getMessage());
}
