#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Cron plugin (maintenance)
 * =========================
 *
 * Returns information about cron runs.
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s)
 * - cron
 *
 * It returns an object with the following key:
 * - string  last_run       Last run in RFC3339/ATOM format, e.g. 2005-08-15T15:52:01+00:00
 * - int     last_run_ago   Last run relative to the current time, in seconds > 0 (e.g. 200)
 * - int     running_count  Cron runs that have status 'running' in the database - most often these are aborted for some reason
 *
 * Interpreting
 * ------------
 * Worth noticing:
 * - last_run_ago > 7200
 * - running_count > 50
 *
 * In a typical Magento installation, cron will run somewhere between every minute and once
 * in 5 minutes.
 */
class Byte_Mavo_Plugin_CheckCron extends Byte_Mavo_Plugin_Abstract
{
    protected $_stubs = array(
        '{"cron":{"last_run":«last_run»,"last_run_ago":«last_run_ago»,"running_count":1}}',
        '{"cron":{"last_run":null,"last_run_ago":null,"running_count":0}}',
        '{"error": "Magento not found"}',
    );


    public function run()
    {
        parent::run();

        $lastRun = $this->getCronLastFinishedAt();

        $this->addResult('cron', array(
            'last_run'      => $lastRun ? date(DATE_ATOM, $lastRun) : null,
            'last_run_ago'  => $lastRun ? time() - $lastRun : null,
            'running_count' => $this->getCronRunningCount(),
        ));
        $this->outputResult();
    }


    /**
     * Get last cron finished time
     *
     * @return int Unix timestamp or NULL if it has never run
     */
    public function getCronLastFinishedAt()
    {
        $this->createMagentoEnvironment();
        $schedules = $this->_getCronScheduleCollection()
            ->addFieldToFilter('status', Mage_Cron_Model_Schedule::STATUS_SUCCESS)
            ->addOrder('finished_at', 'DESC')
            ->setPageSize(1)
            ->load();

        foreach ($schedules as $schedule) {
            if ($schedule->getFinishedAt()) {
                return strtotime($schedule->getFinishedAt());
            }
        }

        return null;
    }

    public function getCronRunningCount()
    {
        $this->createMagentoEnvironment();
        $count = $this->_getCronScheduleCollection()
            ->addFieldToFilter('status', Mage_Cron_Model_Schedule::STATUS_RUNNING)
            ->getSize();

        return $count;
    }

    protected function _getCronScheduleCollection()
    {
        $collection = Mage::getResourceModel('cron/schedule_collection');
        if (!$collection || !($collection instanceof Varien_Data_Collection_Db)) {
            throw new Exception('Cron schedule collection cannot be retrieved');
        }
        return $collection;
    }

    protected function _getStub()
    {
        $lastRun = strtotime('-2 hours') + rand(-20, 20) * 60;
        $markers = array(
            '«last_run»' => json_encode(date(DATE_ATOM, $lastRun)),
            '«last_run_ago»' => json_encode(time() - $lastRun),
        );

        // replace markers in all stubs
        for ($i = 0; $i < count($this->_stubs); $i++) {
            $this->_stubs[$i] = strtr($this->_stubs[$i], $markers);
        }

        return parent::_getStub();
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_CheckCron();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_CheckCron::dieError('Error running plugin: ' . $e->getMessage());
}
