#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * HTTP Headers plugin (performance)
 * =================================
 *
 * Checks compression of static and dynamic files
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 * --base_url       [required] The base url of the store (corresponding to the magento root)
 * --skip_homepage  Skip homepage check
 *
 * Output
 * ------
 * Output key(s)
 * - http_encoding
 *
 * Output is an object with keys (file groups):
 * - css
 * - js
 * - home_page
 *
 * The plugin returns for every file group:
 * - bool   compressed  Is the output compressed (using deflate/gzip)
 * - string file        The file it checked (not for file group 'home_page')
 * - string url         The URL it called to retrieve the file
 *
 * Interpreting
 * ------------
 * Uncompressed file groups are worth noticing.
 */
class Byte_Mavo_Plugin_HttpHeaders extends Byte_Mavo_Plugin_Abstract
{
    protected $_stub = '{"http_encoding":{"css":{"compressed":true,"file":"\/skin\/frontend\/default\/default\/css\/oauth-simple.css","url":"http:\/\/shop.byte.nl\/skin\/frontend\/default\/default\/css\/oauth-simple.css"},"js":{"compressed":false,"file":"\/js\/mage\/captcha.js","url":"http:\/\/shop.byte.nl\/js\/mage\/captcha.js"},"home_page":{"compressed":true,"url":"http:\/\/shop.byte.nl\/"}}}';

    public function run()
    {
        parent::run();

        $this->_addLongOpt('base_url:');
        $this->_addLongOpt('skip_homepage');
        $this->addResult('http_encoding', $this->getHttpEncoding());
        $this->outputResult();
    }

    /**
     * Get the base_url parameter, stripped from trailing slash
     *
     * @return string
     */
    protected function _optionBaseUrl()
    {
        if (!($baseUrl = $this->getOpt('base_url'))) {
            $this->dieError('No base url given. Use --base_url=http://magentohost/');
        }
        return rtrim($baseUrl, '/');
    }

    public function getHttpEncoding()
    {
        $result = array();

        $cssFiles = array(
            '/js/mage/*.css',
            '/js/extjs/css/*.css',
            '/js/calendar/*.css',
            '/skin/frontend/default/default/css/*.css',
        );
        $result['css'] = $this->_checkStaticFileCompression($cssFiles);

        $jsFiles = array(
            '/js/mage/*.js',
            '/skin/frontend/base/default/js/*.js',
            '/js/extjs/*.js',
            '/js/prototype/*.js',
        );
        $result['js'] = $this->_checkStaticFileCompression($jsFiles);

        if ($this->getOpt('skip_homepage') !== false) {
            $homePageUrl = $this->_optionBaseUrl() . '/';
            $homePageCompressed = $this->_isUrlCompressed($homePageUrl);
            $result['home_page'] = array(
                'compressed' => $homePageCompressed,
                'url' => $homePageUrl,
            );
        }

        return $result;
    }


    /**
     * Check the compression of static files
     *
     * @param array $patterns  Array of patterns to check - returns the result of the first file found that matches one of the patterns
     * @return array  Associative array with keys bool 'compressed', string 'file' and string 'url' - 'compressed' being string'unknown' if no file is found
     */
    protected function _checkStaticFileCompression(array $patterns)
    {
        if ($file = $this->_findFirstFileUsingPatterns($patterns)) {
            $url = $this->_optionBaseUrl() . $file;
            if ($this->_isUrlCompressed($url)) {
                return array(
                    'compressed' => true,
                    'file'       => $file,
                    'url'        => $url,
                );
            } else {
                return array(
                    'compressed' => false,
                    'file'       => $file,
                    'url'        => $url,
                );
            }
        } else {
            return array(
                'compressed' => 'unknown',
            );
        }

        return $result;
    }


    /**
     * Check the Content-Encoding http header
     *
     * @param string $url
     * @return bool TRUE if the content is compressed, FALSE if not
     */
    protected function _isUrlCompressed($url)
    {
        $headers = $this->_getHttpHeaders($url, true);
        if (!empty($headers['content_encoding']) && in_array(strtolower($headers['content_encoding']), array('gzip', 'deflate'))) {
            return true;
        }
        return false;
    }


    /**
     * Takes an array of patterns and returns the first existing file
     *
     * This function does NOT recurse
     *
     * @param  array   $patterns  Array of patterns, relative to magento path (e.g. array('/index.*'))
     * @return string  First file found or NULL if not found (again, relative to magento path: starting with /)
     */
    protected function _findFirstFileUsingPatterns($patterns)
    {
        $magentoPath = $this->checkMagentoPathArgument();
        foreach ($patterns as $pattern) {
            if ($foundFiles = glob($magentoPath . $pattern)) {
                $file = reset($foundFiles);
                if (substr($file, 0, strlen($magentoPath)) == $magentoPath) {
                    $file = substr($file, strlen($magentoPath));
                }
                return $file;
            }
        }
        return null;
    }


    /**
     * Gets the http headers of a file, without actually downloading the body
     *
     * @param  string  $url  The URL to check
     * @param  bool    $useCompressionHeader  Send a 'Accept-Encoding' header with all supported encodings
     * @return array Array of name => value pairs
     */
    protected function _getHttpHeaders($url, $useCompressionHeader = true)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_FILETIME, true);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        if ($useCompressionHeader) {
            curl_setopt($ch, CURLOPT_ENCODING, '');
        }
        $headerString = curl_exec($ch);

        $lines = preg_split('/[\r\n]+/', $headerString);
        foreach ($lines as $line) {
            if (strpos($line, ':') !== false) {
                list($key, $value) = explode(":", $line, 2);
                $headers[preg_replace('/[^a-z0-9]/', '_', strtolower($key))] = trim($value);
            }
        }
        return $headers;
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_HttpHeaders();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_HttpHeaders::dieError('Error running plugin: ' . $e->getMessage());
}
