#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Logtables plugin (maintenance)
 * ==============================
 *
 * Checks log cleaning setting and log table sizes
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s)
 * - log_cleaning
 * - log_tables
 *
 * Returns for log_cleaning an object with keys:
 * - bool    enabled            Is log cleaning enabled?
 * - int     clean_after_days   How many days of data must be kept after cleaning?
 *
 * Returns an object for log_tables with the keys being the internal Magento identifiers
 * for the tables and the values being objects with the following keys:
 * - string  table    The table name in the database
 * - int     rows     Number of rows
 * - int     data     Data size in bytes (as reported by MySQL SHOW TABLE STATUS)
 *
 * Interpreting
 * ------------
 * - Log cleaning must be on (but has no effect if the cron is not running)
 * - Difficult to tell what a large number of rows is - it all depends on the number
 *   of visitors and the setting of 'clean after days'. In most cases, 1.000.000+ rows
 *   is too much.
 */
class Byte_Mavo_Plugin_Parameters extends Byte_Mavo_Plugin_Abstract
{
    const XML_PATH_LOG_CLEAN_ENABLED = 'system/log/enabled';
    const XML_LOG_CLEAN_DAYS         = 'system/log/clean_after_day';

    protected $_stub = '{
  "log_cleaning":{
    "enabled":false,
    "clean_after_days":"180"
  },
  "log_tables":{
    "log\/customer":{
      "table":"log_customer",
      "rows":"0",
      "data":"16384"
    },
    "log\/visitor":{
      "table":"log_visitor",
      "rows":"608",
      "data":"98304"
    },
    "log\/visitor_info":{
      "table":"log_visitor_info",
      "rows":"951",
      "data":"81920"
    },
    "log\/url_table":{
      "table":"log_url",
      "rows":"866",
      "data":"65536"
    },
    "log\/url_info_table":{
      "table":"log_url_info",
      "rows":"919",
      "data":"131072"
    },
    "log\/quote_table":{
      "table":"log_quote",
      "rows":"0",
      "data":"16384"
    },
    "reports\/viewed_product_index":{
      "table":"report_viewed_product_index",
      "rows":"313",
      "data":"16384"
    },
    "reports\/compared_product_index":{
      "table":"report_compared_product_index",
      "rows":"0",
      "data":"16384"
    },
    "reports\/event":{
      "table":"report_event",
      "rows":"325",
      "data":"16384"
    },
    "catalog\/compare_item":{
      "table":"catalog_compare_item",
      "rows":"0",
      "data":"16384"
    }
  }
}
';


    public function run()
    {
        parent::run();

        $this->createMagentoEnvironment();

        $this->addResult('log_cleaning', $this->getLogCleaningEnabled());
        $this->addResult('log_tables',   $this->getLogTables());
        $this->outputResult();
    }


    public function getLogCleaningEnabled()
    {
        return array(
            'enabled'          => Mage::getStoreConfigFlag(self::XML_PATH_LOG_CLEAN_ENABLED),
            'clean_after_days' => Mage::getStoreConfig(self::XML_LOG_CLEAN_DAYS),
        );
    }


    public function getLogTables()
    {
        $log = Mage::getModel('log/log');

        $resource = $log->getResource();
        $adapter  = $resource->getReadConnection();

        // log tables
        $tables = array(
            'log/customer',
            'log/visitor',
            'log/visitor_info',
            'log/url_table',
            'log/url_info_table',
            'log/quote_table',

            'reports/viewed_product_index',
            'reports/compared_product_index',
            'reports/event',

            'catalog/compare_item',
        );

        $result = array();
        foreach ($tables as $table) {
            $tableName = $resource->getTable($table);

            $query  = $adapter->quoteInto('SHOW TABLE STATUS LIKE ?', $tableName);
            $status = $adapter->fetchRow($query);
            if (!$status) {
                continue;
            }

            $result[$table] = array(
                'table' => $tableName,
                'rows'  => $status['Rows'],
                'data'  => $status['Data_length'],
            );
        }

        return $result;
    }
}

try {
    $plugin = new Byte_Mavo_Plugin_Parameters();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Parameters::dieError('Error running plugin: ' . $e->getMessage());
}
