#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Media plugin (maintenance)
 * ==========================
 *
 * Outputs information about media files on disk and in the database.
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s)
 * - media
 *
 * Output is an object with keys:
 * - string[]  missing_in_db      Array of files that are on disk, but are not referenced to in the database
 * - string[]  missing_from_disk  Array of files that are in the database, but don't exist on disk
 * - int       unused_percentage  Percentage of media files on disk that are not in the database
 * - int       total_disk         Number of media files on disk (in media/catalog/product)
 * - int       total_database     Number of media files in the database
 *
 * Interpreting
 * ------------
 * Worth noticing:
 * - unsused_percentage > 10
 */
class Byte_Mavo_Plugin_Media extends Byte_Mavo_Plugin_Abstract
{
    // this is the output of Magento demo data
    protected $_stub = '{
  "media":{
    "missing_in_db":[
      "\/a\/r\/argus-qc-2185-quick-click-5mp-digital-camera-3.jpg",
      "\/a\/r\/argus-qc-2185-quick-click-5mp-digital-camera-4.jpg",
      "\/a\/t\/at-t-8525-pda-2.jpg",
      "\/a\/t\/at-t-8525-pda-3.jpg",
      "\/b\/l\/blackberry-8100-pearl-3.jpg",
      "\/b\/l\/blackberry-8100-pearl-4.jpg",
      "\/c\/a\/canon-powershot-a630-8mp-digital-camera-with-4x-optical-zoom-3.jpg",
      "\/c\/a\/canon-powershot-a630-8mp-digital-camera-with-4x-optical-zoom-4.jpg",
      "\/c\/h\/chair-1.jpg",
      "\/c\/h\/chair-2.jpg",
      "\/c\/o\/couch-1.jpg",
      "\/c\/o\/couch-2.jpg",
      "\/k\/o\/kodak-easyshare-c530-5mp-digital-camera-3.jpg",
      "\/k\/o\/kodak-easyshare-c530-5mp-digital-camera-4.jpg",
      "\/m\/a\/magento-red-furniture-set-1.jpg",
      "\/m\/a\/magento-red-furniture-set-2.jpg",
      "\/n\/o\/nokia-2610-phone-3.jpg",
      "\/n\/o\/nokia-2610-phone-4.jpg",
      "\/o\/l\/olympus-stylus-750-7-1mp-digital-camera-3.jpg",
      "\/o\/l\/olympus-stylus-750-7-1mp-digital-camera-4.jpg",
      "\/o\/t\/ottoman-1.jpg",
      "\/o\/t\/ottoman-2.jpg",
      "\/s\/a\/samsung-mm-a900m-ace-1.jpg",
      "\/s\/a\/samsung-mm-a900m-ace-2.jpg",
      "\/s\/l\/slr-camera-tripod-1.jpg",
      "\/s\/l\/slr-camera-tripod-2.jpg",
      "\/s\/o\/sony-ericsson-w810i-3.jpg",
      "\/s\/o\/sony-ericsson-w810i-4.jpg",
      "\/t\/e\/test-phone-1.jpg",
      "\/t\/e\/test-phone-2.jpg",
      "\/t\/e\/test-phone-3.jpg",
      "\/t\/e\/test-phone.jpg",
      "\/t\/o\/toshiba-m285-e-14-1.jpg",
      "\/t\/o\/toshiba-m285-e-14-2.jpg"
    ],
    "missing_from_disk":[
      "\/a\/n\/anashria-womens-premier-leather-sandal-3.jpg",
      "\/a\/n\/anashria-womens-premier-leather-sandal-4.jpg"
    ],
    "unused_percentage":23,
    "total_disk":145,
    "total_database":113
  }
}
';


    public function run()
    {
        parent::run();

        $this->addResult('media', $this->getMedia());
        $this->outputResult();
    }


    public function getMedia()
    {
        $this->createMagentoEnvironment();
        $this->ensureMagentoCommunityVersion('1.4.0.0');

        $mediaDir = Mage::getBaseDir('media') . '/catalog/product';
        chdir($mediaDir);
        exec('find -type f', $diskFiles);
        $diskFiles = array_unique($diskFiles);
        // now remove /cache/* entries and format the file names the same as in the database
        foreach ($diskFiles as $key => $file) {
            if (substr($file, 0, 8) == './cache/') {
                unset($diskFiles[$key]);
                continue;
            }
            if (substr($file, 0, 2) == './') {
                // replace './' by '/'
                $diskFiles[$key] = substr($file, 1);
            }
        }

        /** @var Mage_Core_Model_Resource */
        $resource = Mage::getSingleton('core/resource');
        $connection = $resource->getConnection('read');
        $select = $connection->select()
            ->from(array('mg' => $resource->getTableName('catalog/product_attribute_media_gallery')), array('value'));
        $dbFiles = array_unique($connection->fetchCol($select));

        sort($dbFiles);
        sort($diskFiles);

        return array(
            'missing_in_db'     => ($unused = array_values(array_diff($diskFiles, $dbFiles))),
            'missing_from_disk' => array_values(array_diff($dbFiles, $diskFiles)),
            'unused_percentage' => round((count($unused) / count($diskFiles)) * 100),
            'total_disk'        => count($diskFiles),
            'total_database'    => count($dbFiles),
        );
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_Media();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Media::dieError('Error running plugin: ' . $e->getMessage());
}
