#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Readable plugin (security)
 * ==========================
 *
 * Checks web readability of files/paths that should not be readable.
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 * --base_url       [required] The base url of the store (corresponding to the magento root)
 *
 * Output
 * ------
 * Output key(s)
 * - readable
 *
 * Output is an object with keys (groups):
 * - var              var/ directory should not be readable (cache contains sensitive information)
 * - app_etc          app/etc directory should not be readable (it contains database login information)
 * - release_notes    release notes file reveals installed Magento version
 *
 * Every group itself is an object with keys:
 * - bool   exists    If the path/file exists
 * - bool   readable  If this group seems readable (it checks only the first file it can find)
 * - string file      The file it checked (relative to --magentopath, e.g. /app/etc/local.xml)
 * - string file      The url it checked
 *
 * Interpreting
 * ------------
 * Every group with readable == true is worth noticing (alarming!)
 */
class Byte_Mavo_Plugin_Readable extends Byte_Mavo_Plugin_Abstract
{
    protected $_stub = '
        {
          "readable":{
            "var":{
              "readable":false,
              "exists":true,
              "file":"\/var\/resource_config.json",
              "url":"http:\/\/shop.byte.nl\/var\/resource_config.json"
            },
            "app_etc":{
              "readable":false,
              "exists":true,
              "file":"\/app\/etc\/local.xml.additional",
              "url":"http:\/\/shop.byte.nl\/app\/etc\/local.xml.additional"
            },
            "release_notes":{
              "readable":true,
              "exists":true,
              "file":"\/RELEASE_NOTES.txt",
              "url":"http:\/\/shop.byte.nl\/RELEASE_NOTES.txt"
            }
          }
        }
    ';


    public function run()
    {
        parent::run();

        $this->_addLongOpt('base_url:');
        $this->addResult('readable', $this->getReadable());
        $this->outputResult();
    }


    /**
     * Get the base_url parameter, stripped from trailing slash
     *
     * @return string
     */
    protected function _optionBaseUrl()
    {
        if (!($baseUrl = $this->getOpt('base_url'))) {
            $this->dieError('No base url given. Use --base_url=http://magentohost/');
        }
        return rtrim($baseUrl, '/');
    }


    public function getReadable()
    {
        $magentoPath = $this->checkMagentoPathArgument();
        $result = array();
        foreach (array('var' => '/var/', 'app_etc' => '/app/etc/', 'release_notes' => '/RELEASE_NOTES.txt') as $key => $path) {
            if (file_exists($magentoPath . '/' . $path)) {
                if (substr($path, -1) == '/') {
                    $path = $this->_findFirstFile($magentoPath . '/' . $path);
                    // now strip base path
                    $path = substr($path, strlen($magentoPath) + 1);
                }

                $baseUrl = $this->_optionBaseUrl();
                $url = $baseUrl . $path;
                if ($this->_isReadable($url)) {
                    $result[$key] = array(
                        'readable' => true,
                        'exists' => true,
                        'file' => $path,
                        'url'  => $url,
                    );
                } else {
                    $result[$key] = array(
                        'readable' => false,
                        'exists' => true,
                        'file' => $path,
                        'url'  => $url,
                    );
                }
            } else {
                $result[$key] = array('exists' => false);
            }
        }
        return $result;
    }


    /**
     * Recursively finds the first regular file in a directory
     *
     * @param string $dir  Directory, with or without trailing slash
     * @return string  Path of the first regular file, or FALSE if no regular file found
     */
    protected function _findFirstFile($dir)
    {
        $dir = rtrim($dir, '/');
        if (is_dir($dir)) {
            $directories = array();
            $dh = opendir($dir);
            while (($file = readdir($dh)) !== false) {
                if (substr($file, 0, 1) == '.') {
                    continue;
                }
                if (is_file($dir . '/' . $file)) {
                    closedir($dh);
                    return $dir . '/' . $file;
                }
                if (is_dir($dir . '/' . $file)) {
                    // save for later processing
                    // don't process immediately: the lower the level the better
                    $directories[] = $dir . '/' . $file;
                }
            }
            closedir($dh);
            if ($directories) {
                foreach ($directories as $directory) {
                    if ($result = $this->_findFirstFile($directory)) {
                        return $result;
                    }
                }
            }
        }
        return false;
    }


    /**
     * Checks whether a path is readable via a web call
     *
     * It only checks the HTTP Response code and returns TRUE if the response code = 200
     *
     * @return bool  TRUE if readable, FALSE if not
     */
    protected function _isReadable($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $returnCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        return $returnCode == 200;
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_Readable();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Readable::dieError('Error running plugin: ' . $e->getMessage());
}
