#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Settings plugin (performance)
 * =============================
 *
 * Checks some settings that affect Magento performance.
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s)
 * - global_settings
 *
 * Returns an object with keys:
 * - bool  flat_catalog_category  Are flat categories enabled?
 * - bool  flat_catalog_product   Are flat products enabled?
 * - bool  merge_css              Is CSS merging enabled?
 * - bool  merge_js               Is Javascript merging enabled?
 * - str   insecure_base_url      Base URL for insecure pages
 * - str   secure_base_url        Base URL for secure (SSL) pages
 * - str   admin_url              The full URL to the admin interface
 *
 * Interpreting
 * ------------
 * Every setting that is turned off is worth noticing
 */
class Byte_Mavo_Plugin_Settings extends Byte_Mavo_Plugin_Abstract
{
    protected $_stubs = array(
        '{"global_settings":{"flat_catalog_category":true,"flat_catalog_product":false,"merge_css":false,"merge_js":false,"insecure_base_url":"http://www.domein.nl","secure_base_url":"https://www.domein.nl","admin_url":"https://www.domein.nl/admin"}}',
        '{"global_settings":{"flat_catalog_category":false,"flat_catalog_product":true,"merge_css":true,"merge_js":false,"insecure_base_url":"http://www.domein.nl","secure_base_url":"http://www.domein.nl","admin_url":"http://www.domein.nl/secret_admin"}}',
        '{"error": "Magento not found"}',
    );


    public function run()
    {
        parent::run();

        $this->addResult('global_settings', $this->getSettings());
        $this->outputResult();
    }


    public function getSettings()
    {
        $this->createMagentoEnvironment();

        $flagSettings = array(
            'flat_catalog_category' => 'catalog/frontend/flat_catalog_category',
            'flat_catalog_product'  => 'catalog/frontend/flat_catalog_product',
            'merge_css'             => 'dev/css/merge_css_files',
            'merge_js'              => 'dev/js/merge_files',
        );
        $valueSettings = array(
            'insecure_base_url'     => 'web/unsecure/base_url',
            'secure_base_url'       => 'web/secure/base_url',
        );
        $result = array();
        foreach ($flagSettings as $key => $path) {
            $result[$key] = Mage::getStoreConfigFlag($path, 0);
        }
        foreach ($valueSettings as $key => $path) {
            $result[$key] = Mage::getStoreConfig($path, 0);
        }
        $result['admin_url'] = $this->getAdminURL();
        return $result;
    }

    private function getAdminURL() {
        /*
         * The admin path can be configured in the backend (stored in the database and local.xml) or just
         * in local.xml. This means that getStoreConfig won't cover all cases. Mage::getURL on 'adminhtml'
         * is the only (known) transparant way to retrieve it. Drawback: it adds the script name to the
         * URL. We need to replace the scriptname (check_settings.php) with something generic. This is
         * also done in Magento's cron.php; it seems a valid workaround. Full discussion:
         * https://github.com/ByteInternet/byte-mavo/commit/54ee0fd9544e11f8387010ca03c54be1cceaa19f
         */
        return str_replace(basename(__FILE__), 'index.php', Mage::getUrl('adminhtml', array('_store' => 0)));
    }
}

try {
    $plugin = new Byte_Mavo_Plugin_Settings();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Settings::dieError('Error running plugin: ' . $e->getMessage());
}
