#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Storage plugin (performance)
 * ============================
 *
 * Returns information about where Magento stores its data
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 *
 * Output
 * ------
 * Output key(s)
 * - storage
 *
 * Returns object with keys:
 * - cache
 * - sessions
 *
 * For cache, it returns an object with keys:
 * - backend       Any known or recognized backend (redis,memcached,file,database) or class name (e.g. Byte_Cache_Backend_Trashbin)
 * - slow_backend  [optional] If two level cache is in use, this key returns the slow backend ('backend' being the fast backend)
 *
 * For sessions, it returns an object with keys:
 * - backend       Any known or recognized backend (redis,memcached,files,db) or class name (e.g. Byte_Cache_Backend_Trashbin)
 *
 * !!BEWARE!! cache and sessions both use different namings for file and database storage!
 *
 * Interpreting
 * ------------
 * I think nothing is really worth noticing as long as the site is running on a single
 * node. The only thing that really *can* have a huge negative effect on performance, is
 * using a backend that doesn't support tags. But solving it requires a decent developer.
 */
class Byte_Mavo_Plugin_Storage extends Byte_Mavo_Plugin_Abstract
{
    protected $_stubs = array(
        '{"storage":{"cache":{"backend":"memcached","slow_backend":"file"},"sessions":{"backend":"files"}}}',
        '{"storage":{"cache":{"backend":"file"},"sessions":{"backend":"redis"}}}',
        '{"storage":{"cache":{"backend":"redis"},"sessions":{"backend":"redis"}}}',
        '{"error": "Magento not found"}',
    );

    protected $_backend_names = array(
        'file' => 'files',
        'files' => 'files',
        'cm_cache_backend_file' => 'files',
        'zend_cache_backend_file' => 'files',
        'cm_cache_backend_redis' => 'redis',
        'extendware_ewcachebackend_model_cache_redis' => 'redis',
        'mage_cache_backend_redis' => 'redis',
        'zend_cache_backend_memcached' => 'memcached',
        'zend_cache_backend_libmemcached' => 'memcached',
        'varien_cache_backend_memcached' => 'memcached',
        'varien_cache_backend_database' => 'mysql',
        'database' => 'mysql',
        'db' => 'mysql',
    );

    public function run()
    {
        parent::run();

        $this->createMagentoEnvironment();

        $this->addResult('storage', array(
            'cache' => $this->getCache(),
            'sessions' => $this->getSessions()
        ));
        $this->outputResult();
    }

    protected function getName($cache) {
        if (!$cache) {
            return 'files';
        }
        if (is_object($cache)) {
            $cache = get_class($cache);
        }
        $cache = strtolower($cache);
        if (array_key_exists($cache, $this->_backend_names)) {
            return $this->_backend_names[$cache];
        }
        return $cache;
    }

    /**
     * Return information about cache storage
     *
     * @return array  Key/value pairs with info
     */
    public function getCache()
    {
        $result = array();

        $cache = Mage::app()->getCache()->getBackend();
        if ($cache instanceof Zend_Cache_Backend_TwoLevels) {

            // Break protected property of backends to inspect class name of them
            $twolevels = new ReflectionClass($cache);
            $fb = $twolevels->getProperty('_fastBackend');
            $sb = $twolevels->getProperty('_slowBackend');
            $fb->setAccessible(true);
            $sb->setAccessible(true);
 
            $result = array(
                'backend' => $this->getName($fb->getValue($cache)),
                'slow_backend' => $this->getName($sb->getValue($cache)),
            );
        } else {
            $result = array(
                'backend' => $this->getName($cache),
            );
        }

        return $result;
    }

    /**
     * Return information about session storage
     *
     * @return array  Key/value pairs with info
     */
    public function getSessions()
    {
        $backend = (string) Mage::getConfig()->getNode('global/session_save');
        return array('backend' => $this->getName($backend));
    }
}

try {
    $plugin = new Byte_Mavo_Plugin_Storage();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_Storage::dieError('Error running plugin: ' . $e->getMessage());
}
