#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Var Dir plugin (maintenance)
 * ============================
 *
 * Returns information about reports and log files
 *
 * Command line arguments
 * ----------------------
 * --magentopath              [required] Path to magento root
 * --file_age_recent_seconds  Defines what is 'recent' in seconds (defaults to 21600 / 6h)
 * --file_size_large_file     Defines what is a 'large file' in bytes (defaults to 52428800 / 50MB)
 *
 * Output
 * ------
 * Output key(s) (separate because these two are really different things)
 * - var_log
 * - var_report
 *
 * Returns object with keys for var_log:
 * - int       size         Total size of var/log/
 * - string[]  large_files  Array of filenames of files which are classified as 'large'
 *
 * Returns object with keys for var_report:
 * - int  file_count     The total number of files (reports)
 * - int  recent_count   The number of 'recent' files (see --file_age_recent_seconds)
 * - int  24hours_count  The number of files that are not older than 24 hours
 *
 * Interpreting
 * ------------
 * Worth noticing:
 * - var_log/size > 100MB (104857600)
 * - var_report/file_count > 100
 * - var_report/recent_count > 6
 */
class Byte_Mavo_Plugin_VarDir extends Byte_Mavo_Plugin_Abstract
{
    protected $_stubs = array(
        '{"var_log":{"size":10339845,"large_files":["system.log"]},"var_report":{"file_count":1283,"recent_count":3,"24hours_count":3}}',
        '{"var_log":{"size":547134691,"large_files":["system.log","error.log"]},"var_report":{"error":"directory does not exist"}}',
        '{"var_log":{"error": "directory does not exist"},"var_report":{"file_count":320,"recent_count":210,"24hours_count":320}}',
        '{"error": "Magento not found"}',
    );

    const FILE_AGE_RECENT_SECONDS = 21600; // 6h
    const FILE_SIZE_LARGE_FILE = 52428800; // 50MB

    
    protected function _construct()
    {
        $this->_addLongOpt('file_age_recent_seconds:');
        $this->_addLongOpt('file_size_large_file:');
    }

    public function run()
    {
        parent::run();

        $this->addResult('var_log', $this->getVarLog());
        $this->addResult('var_report', $this->getVarReport());
        $this->outputResult();
    }

    protected function _optionFileAgeRecentSeconds()
    {
        return $this->getOpt('file_age_recent_seconds', self::FILE_AGE_RECENT_SECONDS);
    }

    protected function _optionFileSizeLargeFile()
    {
        return $this->getOpt('file_size_large_file', self::FILE_SIZE_LARGE_FILE);
    }


    public function getVarLog()
    {
        // this is terribly slow
        // $this->createMagentoEnvironment();
        // $logDir = Mage::getBaseDir('var') . DS . 'log';

        $logDir = $this->checkMagentoPathArgument() . '/var/log';
        $size = 0;
        $largeFiles = array();
        if (is_dir($logDir)) {
            // return value format: '714099  path/to/magento/log/dir' - we'll only use the number of bytes by calling intval()
            $size = intval(exec('du -sb ' . escapeshellarg($logDir)));

            // determine large files
            if ($size > $this->_optionFileSizeLargeFile()) {
                foreach (glob($logDir . '/*') as $file) {
                    if (filesize($file) > $this->_optionFileSizeLargeFile()) {
                        $largeFiles[] = basename($file);
                    }
                }
            }
        }

        return array('size' => $size, 'large_files' => $largeFiles);
    }

    public function getVarReport()
    {
        $reportDir = $this->checkMagentoPathArgument() . '/var/report';
        $fileCount = 0;

        if (is_dir($reportDir)) {
            $fileCount = intval(exec($q = 'find ' . escapeshellarg($reportDir) . ' -type f | wc -l'));
        }

        $ageStatistics = array();
        if ($fileCount > 0) {
            $ageStatistics = $this->_getAgeStatistics(glob($reportDir . '/*'));
        }

        return array(
            'file_count' => $fileCount,
            'recent_count' => !empty($ageStatistics['recent']) ? $ageStatistics['recent'] : 0,
            '24hours_count' => !empty($ageStatistics['24hours']) ? $ageStatistics['24hours'] : 0,
        );
    }


    /**
     * Returns statistics about age of an array of files
     *
     * @param array $files  Array of file names that are assumed to exist (there is no checking here)
     * @return array  Associative array with keys 'recent' and '24hours', both with file counts
     */
    protected function _getAgeStatistics($files)
    {
        $now = time();

        $recentCount = 0;
        $dayCount = 0;
        foreach ($files as $file) {
            if (time() - filemtime($file) <= $this->_optionFileAgeRecentSeconds()) {
                $recentCount++;
                $dayCount++;
                continue;
            }
            if (time() - filemtime($file) <= 86400) {
                $dayCount++;
            }
        }

        return array(
            'recent' => $recentCount,
            '24hours' => $dayCount,
        );
    }

}

try {
    $plugin = new Byte_Mavo_Plugin_VarDir();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_VarDir::dieError('Error running plugin: ' . $e->getMessage());
}
