#!/usr/bin/php -d opcache.enable_cli=0
<?php

require_once 'Abstract.php';

/**
 * Cache plugin to alter settings
 * ==========================
 *
 * Enables of disables Magento cache types. Will change anything that is returned by Magento. At the moment these are:
 *   - config
 *   - layout
 *   - block_html
 *   - translate
 *   - collections
 *   - eav
 *   - config_api
 *   - config_api2
 *
 * Command line arguments
 * ----------------------
 * --magentopath    [required] Path to magento root
 * --cachetype      [required] Type of cache to enable or disable, may be specified more than once for additional types
 * --enable         Enable the specified caches
 * --disable        Disable the specified caches
 *
 * Of course, one of --enable and --disable is required, and specifying both results in an error.
 *
 * Output
 * ------
 * Output key(s):
 * - changed_caches
 *
 * For every changed cachetype, it outputs the new setting: 0 or 1.
 *
 * NB
 * ----
 * We want to change these settings through the internal Magento API, because Magento caches the cache-settings. This means that merely changing them in the database is not enough, but one also must clear the cache. The internal API let's us clear only certain tags, so the normal cache stays intact.
 */
class Byte_Mavo_Plugin_SetCache extends Byte_Mavo_Plugin_Abstract
{
    protected $_stubs = array(
        '{"changed_caches":{"config":1,"translate":1}}',
        '{"changed_caches":{"block_html":0}}',
        '{"changed_caches":{}}',
    );

    
    public function __construct()
    {
        parent::__construct();

        $this->_addLongOpt('cachetype:');
        $this->_addLongOpt('enable');
        $this->_addLongOpt('disable');

        $opts = $this->getOpt();

        if (isset($opts['enable']) && isset($opts['disable'])) {
            $this->dieError('Can\'t enable and disable cache at the same time');
        }
        if (!isset($opts['enable']) && !isset($opts['disable'])) {
            $this->dieError('Specify whether to --enable or to --disable the cache');
	    }
        if (!isset($opts['cachetype'])) {
            $this->dieError('Specify at least one cachetype to enable or disable');
        }
    }

    public function run()
    {
        parent::run();
    
        $this->createMagentoEnvironment();

        $opts = $this->getOpt();
        $types = is_array($opts['cachetype']) ? $opts['cachetype'] : array($opts['cachetype']);
        $result = null;

        if (isset($opts['enable'])) {
            $result = $this->enableCacheTypes($types);
        }
        elseif (isset($opts['disable'])) {
            $result = $this->disableCacheTypes($types);
        }

        $this->addResult('changed_caches', $result);
        $this->outputResult();
    }

    public function enableCacheTypes($types)
    {
        // Inspired by app/code/core/Mage/Adminhtml/controllers/CacheController.php
        $allTypes = Mage::app()->useCache();
        $changed = array();
        $updatedTypes = 0;
        foreach ($types as $code) {
            if (empty($allTypes[$code])) {
                $allTypes[$code] = 1;
                $updatedTypes++;
            }
            // Other than the original code, we will always flush items from the cache. BSTS.
            $tags = Mage::app()->getCacheInstance()->cleanType($code);

	    // We will always return the item with the expected setting, even if it didn't need changing
            $changed[$code] = 1;
        }
        if ($updatedTypes > 0) {
            Mage::app()->saveUseCache($allTypes);
        }
        return $changed;
    }

    public function disableCacheTypes($types)
    {
        // Inspired by app/code/core/Mage/Adminhtml/controllers/CacheController.php
        $allTypes = Mage::app()->useCache();
        $changed = array();
        $updatedTypes = 0;
        foreach ($types as $code) {
            if (!empty($allTypes[$code])) {
                $allTypes[$code] = 0;
                $updatedTypes++;
            }
            $tags = Mage::app()->getCacheInstance()->cleanType($code);

	    // We will always return the item with the expected setting, even if it didn't need changing
            $changed[$code] = 0;
        }
        if ($updatedTypes > 0) {
            Mage::app()->saveUseCache($allTypes);
        }
        return $changed;
    }
}

try {
    $plugin = new Byte_Mavo_Plugin_SetCache();
    $plugin->run();
} catch (Exception $e) {
    Byte_Mavo_Plugin_SetCache::dieError('Error running plugin: ' . $e->getMessage());
}
